@@ -1,10 +1,9 @@
 package Net::DNS::Mailbox;
 
 #
-# $Id: Mailbox.pm 1155 2014-01-05 20:24:17Z willem $
+# $Id: Mailbox.pm 1527 2017-01-18 21:42:48Z willem $
 #
-use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1155 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1527 $)[1];
 
 
 =head1 NAME
@@ -27,6 +26,7 @@ objects representing the DNS coded form of RFC822 mailbox address.
 
 
 use strict;
+use warnings;
 use integer;
 use Carp;
 
@@ -37,11 +37,13 @@ use base qw(Net::DNS::DomainName);
 
 =head2 new
 
-    $mailbox = new Net::DNS::Mailbox('John.Doe@example.com');
-    $mailbox = new Net::DNS::Mailbox('John Doe <j.doe@example.com>');
+    $mailbox = new Net::DNS::Mailbox('John Doe <john.doe@example.com>');
+    $mailbox = new Net::DNS::Mailbox('john.doe@example.com');
+    $mailbox = new Net::DNS::Mailbox('john\.doe.example.com');
 
-Creates a mailbox object which represents the DNS domain encoded form
-of the mail address specified by the character string argument.
+Creates a mailbox object representing the RFC822 mail address specified by
+the character string argument. An encoded domain name is also accepted for
+backward compatibility with Net::DNS 0.68 and earlier.
 
 The argument string consists of printable characters from the 7-bit
 ASCII repertoire.
@@ -57,11 +59,11 @@ sub new {
 	s/>.*$//g;						# strip excess on right
 
 	s/\\\@/\\064/g;						# disguise escaped @
-	s/^(".*)\@(.*")/$1\\064$2/g;				# disguise quoted @
+	s/("[^"]*)\@([^"]*")/$1\\064$2/g;			# disguise quoted @
 
 	my ( $mbox, @host ) = split /\@/;			# split on @ if present
 	for ( $mbox ||= '' ) {
-		s/^"(.*)"/$1/;					# strip quotes
+		s/^.*"(.*)".*$/$1/;				# strip quotes
 		s/\\\./\\046/g;					# disguise escaped dot
 		s/\./\\046/g if @host;				# escape dots in local part
 	}
@@ -81,12 +83,16 @@ in RFC1035 section 8.
 =cut
 
 sub address {
+	return unless defined wantarray;
 	my @label = shift->label;
 	local $_ = shift(@label) || return '<>';
+	s/\\\\//g;						# delete escaped \
+	s/\\\d\d\d//g;						# delete non-printable
 	s/\\\./\./g;						# unescape dots
-	s/\\032/ /g;						# unescape space
-	s/^(.+)$/"$1"/ if /[ ",@\[\\\]]/;			# quote local part
-	return join '@', $_, join( '.', @label ) || ();
+	s/[\\"]//g;						# delete \ "
+	s/^(.*)$/"$1"/ if /["(),:;<>@\[\\\]]/;			# quote local part
+	return $_ unless scalar(@label);
+	join '@', $_, join '.', @label;
 }
 
 
@@ -126,13 +132,29 @@ Copyright (c)2009,2012 Dick Franks.
 
 All rights reserved.
 
-This program is free software; you may redistribute it and/or
-modify it under the same terms as Perl itself.
+
+=head1 LICENSE
+
+Permission to use, copy, modify, and distribute this software and its
+documentation for any purpose and without fee is hereby granted, provided
+that the above copyright notice appear in all copies and that both that
+copyright notice and this permission notice appear in supporting
+documentation, and that the name of the author not be used in advertising
+or publicity pertaining to distribution of the software without specific
+prior written permission.
+
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+DEALINGS IN THE SOFTWARE.
 
 
 =head1 SEE ALSO
 
-L<perl>, L<Net::DNS>, L<Net::DNS::DomainName>, RFC822, RFC1035, RFC5322
+L<perl>, L<Net::DNS>, L<Net::DNS::DomainName>, RFC1035, RFC5322 (RFC822)
 
 =cut
 
