@@ -1,13 +1,13 @@
 package Net::DNS::RR::DHCID;
 
 #
-# $Id: DHCID.pm 1188 2014-04-03 18:54:34Z willem $
+# $Id: DHCID.pm 1528 2017-01-18 21:44:58Z willem $
 #
-use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1188 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1528 $)[1];
 
 
 use strict;
+use warnings;
 use base qw(Net::DNS::RR);
 
 =head1 NAME
@@ -22,7 +22,7 @@ use integer;
 use MIME::Base64;
 
 
-sub decode_rdata {			## decode rdata from wire-format octet string
+sub _decode_rdata {			## decode rdata from wire-format octet string
 	my $self = shift;
 	my ( $data, $offset ) = @_;
 
@@ -31,27 +31,27 @@ sub decode_rdata {			## decode rdata from wire-format octet string
 }
 
 
-sub encode_rdata {			## encode rdata as wire-format octet string
+sub _encode_rdata {			## encode rdata as wire-format octet string
 	my $self = shift;
 
-	return '' unless $self->{digest};
+	return '' unless defined $self->{digest};
 	pack 'nC a*', map $self->$_, qw(identifiertype digesttype digest);
 }
 
 
-sub format_rdata {			## format rdata portion of RR string.
+sub _format_rdata {			## format rdata portion of RR string.
 	my $self = shift;
 
-	my $base64 = MIME::Base64::encode $self->encode_rdata;
-	chomp $base64;
-	return length($base64) > 40 ? "(\n$base64 )" : $base64;
+	my @base64 = split /\s+/, encode_base64( $self->_encode_rdata );
 }
 
 
-sub parse_rdata {			## populate RR from rdata in argument list
+sub _parse_rdata {			## populate RR from rdata in argument list
 	my $self = shift;
 
-	$self->rdata(@_);
+	my $data = MIME::Base64::decode( join "", @_ );
+	my $size = length($data) - 3;
+	@{$self}{qw(identifiertype digesttype digest)} = unpack "n C a$size", $data;
 }
 
 
@@ -78,7 +78,7 @@ sub identifiertype {
 	my $self = shift;
 
 	$self->{identifiertype} = 0 + shift if scalar @_;
-	return $self->{identifiertype} || 0;
+	$self->{identifiertype} || 0;
 }
 
 
@@ -86,7 +86,7 @@ sub digesttype {
 	my $self = shift;
 
 	$self->{digesttype} = 0 + shift if scalar @_;
-	return $self->{digesttype} || 0;
+	$self->{digesttype} || 0;
 }
 
 
@@ -98,17 +98,6 @@ sub digest {
 }
 
 
-sub rdata {
-	my $self = shift;
-
-	if ( scalar @_ ) {
-		my $data = MIME::Base64::decode( join "", @_ );
-		my $size = length($data) - 3;
-		@{$self}{qw(identifiertype digesttype digest)} = unpack "n C a$size", $data;
-	}
-	return MIME::Base64::encode( $self->encode_rdata, "" ) if defined wantarray;
-}
-
 1;
 __END__
 
@@ -116,7 +105,16 @@ __END__
 =head1 SYNOPSIS
 
     use Net::DNS;
-    $rr = new Net::DNS::RR('name DHCID algorithm fptype fingerprint');
+    $rr = new Net::DNS::RR('client.example.com. DHCID ( AAAB
+	xLmlskllE0MVjd57zHcWmEH3pCQ6VytcKD//7es/deY=');
+
+    $rr = new Net::DNS::RR(
+	name	       => 'client.example.com',
+	type	       => 'DHCID',
+	digest	       => 'ObfuscatedIdentityData',
+	digesttype     => 1,
+	identifiertype => 2,
+	);
 
 =head1 DESCRIPTION
 
@@ -155,15 +153,6 @@ algorithm used to obfuscate the DHCP identity information.
 
 Binary representation of the digest of DHCP identity information.
 
-=head2 rdata
-
-The RDATA for this record is stored in master files as a single
-block using Base64 representation.
-
-White space characters may appear anywhere within the Base64 text
-and will be silently ignored.
-
-
 
 =head1 COPYRIGHT
 
@@ -171,12 +160,28 @@ Copyright (c)2009 Olaf Kolkman, NLnet Labs.
 
 All rights reserved.
 
-This program is free software; you may redistribute it and/or
-modify it under the same terms as Perl itself.
-
 Package template (c)2009,2012 O.M.Kolkman and R.W.Franks.
 
 
+=head1 LICENSE
+
+Permission to use, copy, modify, and distribute this software and its
+documentation for any purpose and without fee is hereby granted, provided
+that the above copyright notice appear in all copies and that both that
+copyright notice and this permission notice appear in supporting
+documentation, and that the name of the author not be used in advertising
+or publicity pertaining to distribution of the software without specific
+prior written permission.
+
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+DEALINGS IN THE SOFTWARE.
+
+
 =head1 SEE ALSO
 
 L<perl>, L<Net::DNS>, L<Net::DNS::RR>, RFC4701
