@@ -1,13 +1,13 @@
 package Net::DNS::RR::TSIG;
 
 #
-# $Id: TSIG.pm 1282 2014-10-27 09:45:19Z willem $
+# $Id: TSIG.pm 1561 2017-04-19 13:08:13Z willem $
 #
-use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1282 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1561 $)[1];
 
 
 use strict;
+use warnings;
 use base qw(Net::DNS::RR);
 
 =head1 NAME
@@ -20,21 +20,20 @@ Net::DNS::RR::TSIG - DNS TSIG resource record
 use integer;
 
 use Carp;
-use MIME::Base64;
 
+eval 'require Digest::HMAC';
+eval 'require Digest::MD5';
+eval 'require Digest::SHA';
+eval 'require MIME::Base64';
+
+use Net::DNS::DomainName;
 use Net::DNS::Parameters;
 
-require Net::DNS::DomainName;
-
-require Digest::HMAC;
-require Digest::MD5;
-require Digest::SHA;
-
 use constant ANY  => classbyname qw(ANY);
 use constant TSIG => typebyname qw(TSIG);
 
-
 {
+	# source: http://www.iana.org/assignments/tsig-algorithm-names
 	my @algbyname = (
 		'HMAC-MD5.SIG-ALG.REG.INT' => 157,
 		'HMAC-SHA1'		   => 161,
@@ -49,32 +48,37 @@ use constant TSIG => typebyname qw(TSIG);
 		'HMAC-SHA' => 161,
 		);
 
+
 	my %algbyval = reverse @algbyname;
 
+	my $map = sub {
+		my $arg = shift;
+		return $arg if $arg =~ /^\d/;
+		$arg =~ s/[^A-Za-z0-9]//g;			# strip non-alphanumerics
+		uc($arg);
+	};
 
-	my @algbynum = map { ( $_, 0 + $_ ) } keys %algbyval;	# accept algorithm number
+	my @pairedval = sort ( 1 .. 254, 1 .. 254 );		# also accept number
+	my %algbyname = map &$map($_), @algbyalias, @algbyname, @pairedval;
 
-	my %algbyname = map { s /[^A-Za-z0-9]//g; $_ } @algbyalias, @algbyname, @algbynum;
-
-
-	sub algbyname {
+	sub _algbyname {
 		my $key = uc shift;				# synthetic key
-		$key =~ s /[^A-Z0-9]//g;			# strip non-alphanumerics
-		return $algbyname{$key};
+		$key =~ s/[^A-Z0-9]//g;				# strip non-alphanumerics
+		$algbyname{$key};
 	}
 
-	sub algbyval {
+	sub _algbyval {
 		my $value = shift;
-		return $algbyval{$value} || $value;
+		$algbyval{$value};
 	}
 }
 
 
-
-sub decode_rdata {			## decode rdata from wire-format octet string
+sub _decode_rdata {			## decode rdata from wire-format octet string
 	my $self = shift;
 	my ( $data, $offset ) = @_;
 
+	my $limit = $offset + $self->{rdlength};
 	( $self->{algorithm}, $offset ) = decode Net::DNS::DomainName(@_);
 
 	# Design decision: Use 32 bits, which will work until the end of time()!
@@ -92,13 +96,13 @@ sub decode_rdata {			## decode rdata from wire-format octet string
 	$self->{other} = unpack "\@$offset xx a$other_size", $$data;
 	$offset += $other_size + 2;
 
-	croak('misplaced or corrupt TSIG') unless $offset == length $$data;
-	substr( $$data, $self->{offset} || $offset ) = '';
-	$self->{rawref} = $data;
+	croak('misplaced or corrupt TSIG') unless $limit == length $$data;
+	my $raw = substr $$data, 0, $self->{offset};
+	$self->{rawref} = \$raw;
 }
 
 
-sub encode_rdata {			## encode rdata as wire-format octet string
+sub _encode_rdata {			## encode rdata as wire-format octet string
 	my $self = shift;
 
 	my $macbin = $self->macbin;
@@ -117,34 +121,30 @@ sub encode_rdata {			## encode rdata as wire-format octet string
 
 	$rdata .= pack 'na*', length($macbin), $macbin;
 
-	$rdata .= pack 'nn', $self->original_id, $self->{error} || 0;
+	$rdata .= pack 'nn', $self->original_id, $self->{error};
 
-	my $other = $self->other || '';
+	my $other = $self->other;
 	$rdata .= pack 'na*', length($other), $other;
 
 	return $rdata;
 }
 
 
-sub format_rdata {			## format rdata portion of RR string.
-	my $self = shift;
-
-	my @lines = (
-		join( "\t", 'algorithm:', $self->algorithm ),
-		join( "\t", 'time signed:', $self->time_signed, 'fudge:', $self->fudge ),
-		join( "\t", 'signature:',   $self->mac ),
-		join( "\t", 'original id:', $self->original_id ),
-		join( "\t", $self->error, $self->other || '' ) );
-	join "\n; ", '', @lines;
-}
-
-
-sub defaults() {			## specify RR attribute default values
+sub _defaults {				## specify RR attribute default values
 	my $self = shift;
 
 	$self->algorithm(157);
 	$self->class('ANY');
+	$self->error(0);
 	$self->fudge(300);
+	$self->other('');
+}
+
+
+sub _size {				## estimate encoded size
+	my $self = shift;
+	my $clone = bless {%$self}, ref($self);			   # shallow clone
+	length $clone->encode( 0, undef, new Net::DNS::Packet() );
 }
 
 
@@ -152,8 +152,32 @@ sub encode {				## overide RR method
 	my $self = shift;
 
 	my $kname = $self->{owner}->encode();			# uncompressed key name
-	my $rdata = eval { $self->encode_rdata(@_) } || '';
-	return pack 'a* n2 N n a*', $kname, TSIG, ANY, 0, length $rdata, $rdata;
+	my $rdata = eval { $self->_encode_rdata(@_) } || '';
+	pack 'a* n2 N n a*', $kname, TSIG, ANY, 0, length $rdata, $rdata;
+}
+
+
+sub string {				## overide RR method
+	my $self = shift;
+
+	my $owner	= $self->{owner}->string;
+	my $type	= $self->type;
+	my $algorithm	= $self->algorithm;
+	my $time_signed = $self->time_signed;
+	my $fudge	= $self->fudge;
+	my $signature	= $self->mac;
+	my $original_id = $self->original_id;
+	my $error	= $self->error;
+	my $other	= $self->other;
+
+	return <<"QQ";
+; $owner	$type	
+;	algorithm:	$algorithm
+;	time signed:	$time_signed	fudge:	$fudge
+;	signature:	$signature
+;	original id:	$original_id
+;			$error	$other
+QQ
 }
 
 
@@ -164,7 +188,7 @@ sub key {
 	my $self = shift;
 
 	$self->keybin( MIME::Base64::decode( join "", @_ ) ) if scalar @_;
-	return MIME::Base64::encode( $self->keybin(), "" ) if defined wantarray;
+	MIME::Base64::encode( $self->keybin(), "" ) if defined wantarray;
 }
 
 
@@ -175,7 +199,7 @@ sub time_signed {
 	my $self = shift;
 
 	$self->{time_signed} = 0 + shift if scalar @_;
-	return $self->{time_signed} ||= time();
+	$self->{time_signed} = time() unless $self->{time_signed};
 }
 
 
@@ -183,7 +207,7 @@ sub fudge {
 	my $self = shift;
 
 	$self->{fudge} = 0 + shift if scalar @_;
-	return $self->{fudge} || 0;
+	$self->{fudge} || 0;
 }
 
 
@@ -205,8 +229,9 @@ sub macbin {
 
 sub prior_mac {
 	my $self = shift;
+	my @args = map { /[^0-9A-Fa-f]/ ? croak "corrupt hexadecimal" : $_ } @_;
 
-	$self->prior_macbin( pack "H*", map { die "!hex!" if m/[^0-9A-Fa-f]/; $_ } join "", @_ ) if scalar @_;
+	$self->prior_macbin( pack "H*", join "", @args ) if scalar @args;
 	unpack "H*", $self->prior_macbin() if defined wantarray;
 }
 
@@ -221,8 +246,9 @@ sub prior_macbin {
 
 sub request_mac {
 	my $self = shift;
+	my @args = map { /[^0-9A-Fa-f]/ ? croak "corrupt hexadecimal" : $_ } @_;
 
-	$self->request_macbin( pack "H*", map { die "!hex!" if m/[^0-9A-Fa-f]/; $_ } join "", @_ ) if scalar @_;
+	$self->request_macbin( pack "H*", join "", @args ) if scalar @args;
 	unpack "H*", $self->request_macbin() if defined wantarray;
 }
 
@@ -239,26 +265,26 @@ sub original_id {
 	my $self = shift;
 
 	$self->{original_id} = 0 + shift if scalar @_;
-	return $self->{original_id} || 0;
+	$self->{original_id} || 0;
 }
 
 
 sub error {
 	my $self = shift;
 	$self->{error} = rcodebyname(shift) if scalar @_;
-	rcodebyval( $self->{error} || 0 );
+	rcodebyval( $self->{error} );
 }
 
 
 sub other {
 	my $self = shift;
-
 	$self->{other} = shift if scalar @_;
-	$self->{other} || "";
+	my $time = $self->{error} == 18 ? pack 'xxN', time() : '';
+	$self->{other} = $time unless $self->{other};
 }
 
 
-sub other_data {&other}
+sub other_data { &other; }					# uncoverable pod
 
 
 sub sig_function {
@@ -268,7 +294,7 @@ sub sig_function {
 	$self->{sig_function} = shift;
 }
 
-sub sign_func { &sig_function; }	## historical
+sub sign_func { &sig_function; }				# uncoverable pod
 
 
 sub sig_data {
@@ -280,11 +306,15 @@ sub sig_data {
 		local $message->{additional} = \@unsigned;	# remake header image
 		my @part = qw(question answer authority additional);
 		my @size = map scalar( @{$message->{$_}} ), @part;
-		my $data = $self->{rawref};
-		delete $self->{rawref};
-		my $orig = $data ? $self->original_id : $message->{id};
-		my $hbin = pack 'n6', $orig, $message->{status}, @size;
-		$message = $hbin . substr $data ? $$data : $message->data, length $hbin;
+		if ( my $rawref = $self->{rawref} ) {
+			delete $self->{rawref};
+			my $hbin = pack 'n6', $self->original_id, $message->{status}, @size;
+			$message = join '', $hbin, substr $$rawref, length $hbin;
+		} else {
+			my $data = $message->data;
+			my $hbin = pack 'n6', $message->{id}, $message->{status}, @size;
+			$message = join '', $hbin, substr $data, length $hbin;
+		}
 	}
 
 	# Design decision: Use 32 bits, which will work until the end of time()!
@@ -292,9 +322,8 @@ sub sig_data {
 
 	# Insert the prior MAC if present (multi-packet message).
 	$self->prior_macbin( $self->{link}->macbin ) if $self->{link};
-	if ( my $prior_mac = $self->prior_macbin ) {
-		return pack 'na* a* a*', length($prior_mac), $prior_mac, $message, $time;
-	}
+	my $prior_macbin = $self->prior_macbin;
+	return pack 'na* a* a*', length($prior_macbin), $prior_macbin, $message, $time if $prior_macbin;
 
 	# Insert the request MAC if present (used to validate responses).
 	my $req_mac = $self->request_macbin;
@@ -309,9 +338,9 @@ sub sig_data {
 
 	$sigdata .= $time;
 
-	$sigdata .= pack 'n', $self->{error} || 0;
+	$sigdata .= pack 'n', $self->{error};
 
-	my $other = $self->other || '';
+	my $other = $self->other;
 	$sigdata .= pack 'na*', length($other), $other;
 
 	return $sigdata;
@@ -320,21 +349,37 @@ sub sig_data {
 
 sub create {
 	my $class = shift;
-	my $karg = shift || croak 'argument missing or undefined';
+	my $karg  = shift;
+	croak 'argument undefined' unless defined $karg;
 
 	if ( ref($karg) ) {
-		return $karg->_chain if ref($karg) eq __PACKAGE__;
-		croak "Usage:	create $class( keyfile )\n\tcreate $class( keyname, key )"
-				unless $karg->isa('Net::DNS::Packet');
+		if ( $karg->isa('Net::DNS::Packet') ) {
+			my $sigrr = $karg->sigrr;
+			croak 'no TSIG in request packet' unless defined $sigrr;
+			return new Net::DNS::RR(		# ( request, options )
+				name	       => $sigrr->name,
+				type	       => 'TSIG',
+				algorithm      => $sigrr->algorithm,
+				request_macbin => $sigrr->macbin,
+				@_
+				);
 
-		my $sigrr = $karg->sigrr || die 'no TSIG in request packet';
-		return new Net::DNS::RR(			# ( request, options )
-			name	       => $sigrr->name,
-			type	       => 'TSIG',
-			algorithm      => $sigrr->algorithm,
-			request_macbin => $sigrr->macbin,
-			@_
-			);
+		} elsif ( ref($karg) eq __PACKAGE__ ) {
+			my $tsig = $karg->_chain;
+			$tsig->{macbin} = undef;
+			return $tsig;
+
+		} elsif ( ref($karg) eq 'Net::DNS::RR::KEY' ) {
+			return new Net::DNS::RR(
+				name	  => $karg->name,
+				type	  => 'TSIG',
+				algorithm => $karg->algorithm,
+				key	  => $karg->key,
+				@_
+				);
+		}
+
+		croak "Usage:	create $class(keyfile)\n\tcreate $class(keyname, key)"
 
 	} elsif ( scalar(@_) == 1 ) {
 		my $key = shift;				# ( keyname, key )
@@ -357,7 +402,8 @@ sub create {
 		}
 
 		my ( $vol, $dir, $file ) = File::Spec->splitpath( $keyfile->name );
-		my $kname = $file =~ /^K([^+]+)+.+private$/ ? $1 : undef;
+		my $kname;
+		$kname = $1 if $file =~ /^K([^+]+)+.+private$/;
 		return new Net::DNS::RR(
 			name	  => $kname,
 			type	  => 'TSIG',
@@ -368,16 +414,13 @@ sub create {
 
 	} else {						# ( keyfile, options )
 		require Net::DNS::ZoneFile;
-		my $keyfile = new Net::DNS::ZoneFile($karg);
-		my $keyline = $keyfile->_getline;		# bad news: KEY is in Net::DNS::SEC
-		my ( $kname, $c, $t, $f, $p, $algorithm, @key ) = split /\s+/, $keyline;
-		croak 'key file incompatible with TSIG' unless "$c $t $f $p" eq 'IN KEY 512 3';
-		my $key = join '', @key;
+		my $keyrr = new Net::DNS::ZoneFile($karg)->read;
+		croak 'key file incompatible with TSIG' unless $keyrr->type eq 'KEY';
 		return new Net::DNS::RR(
-			name	  => $kname,
+			name	  => $keyrr->name,
 			type	  => 'TSIG',
-			algorithm => $algorithm,
-			key	  => $key,
+			algorithm => $keyrr->algorithm,
+			key	  => $keyrr->key,
 			@_
 			);
 	}
@@ -390,48 +433,46 @@ sub verify {
 
 	unless ( abs( time() - $self->time_signed ) < $self->fudge ) {
 		$self->error(18);				# bad time
-		$self->other( pack 'xxN', time() );
 		return;
 	}
 
 	if ( scalar @_ ) {
+		my $arg = shift;
 
-		unless ( my $arg = shift ) {
+		unless ( ref($arg) ) {
 			$self->error(16);			# bad sig (multi-packet)
 			return;
+		}
 
-		} elsif ( ref($arg) && $arg->isa('Net::DNS::Packet') ) {
-			my $sigrr = $arg->sigrr;		# query TSIG
-			$self->request_macbin( $sigrr->macbin );
-			$self->error(17) && return unless $self->name eq $sigrr->name;
-			$self->error(17) && return unless lc $self->algorithm eq lc $sigrr->algorithm;
+		my $signerkey = lc( join '+', $self->name, $self->algorithm );
+		if ( $arg->isa('Net::DNS::Packet') ) {
+			my $request = $arg->sigrr;		# request TSIG
+			my $rqstkey = lc( join '+', $request->name, $request->algorithm );
+			$self->error(17) unless $signerkey eq $rqstkey;
+			$self->request_macbin( $request->macbin );
 
-		} elsif ( ref($arg) && $arg->isa(__PACKAGE__) ) {
-			my $sigrr = $arg;			# multi-packet
-			$self->prior_macbin( $sigrr->macbin );
-			$self->error(17) && return unless $self->name eq $sigrr->name;
-			$self->error(17) && return unless lc $self->algorithm eq lc $sigrr->algorithm;
+		} elsif ( $arg->isa(__PACKAGE__) ) {
+			my $priorkey = lc( join '+', $arg->name, $arg->algorithm );
+			$self->error(17) unless $signerkey eq $priorkey;
+			$self->prior_macbin( $arg->macbin );
 
 		} else {
 			croak 'Usage: $tsig->verify( $reply, $query )';
 		}
-
 	}
+	return if $self->{error};
 
 	my $sigdata = $self->sig_data($data);			# form data to be verified
 	my $tsigmac = $self->_mac_function($sigdata);
 	my $tsig    = $self->_chain;
-	$tsig->macbin($tsigmac);
 
 	my $macbin = $self->macbin;
 	my $maclen = length $macbin;
 	my $minlen = length($tsigmac) >> 1;			# per RFC4635, 3.1
-	$self->error(1) && return if $maclen < 10;
-	$self->error(1) && return if $maclen < $minlen;
-	$self->error(1) && return if $maclen > length $tsigmac;
+	$self->error(16) unless $macbin eq substr $tsigmac, 0, $maclen;
+	$self->error(1) if $maclen < $minlen or $maclen < 10 or $maclen > length $tsigmac;
 
-	$self->error(16) && return unless $macbin eq substr $tsigmac, 0, $maclen;
-	return $tsig;
+	return $self->{error} ? undef : $tsig;
 }
 
 sub vrfyerrstr {
@@ -442,7 +483,6 @@ sub vrfyerrstr {
 
 ########################################
 
-
 {
 	my %digest = (
 		'157' => ['Digest::MD5'],
@@ -461,17 +501,17 @@ sub vrfyerrstr {
 
 		if ( my $algname = shift ) {
 
-			unless ( my $digtype = algbyname($algname) ) {
+			unless ( my $digtype = _algbyname($algname) ) {
 				$self->{algorithm} = new Net::DNS::DomainName($algname);
 
 			} else {
-				$algname = algbyval($digtype);
+				$algname = _algbyval($digtype);
 				$self->{algorithm} = new Net::DNS::DomainName($algname);
 
 				my ( $hash, @param ) = @{$digest{$digtype}};
 				my ( undef, @block ) = @param;
+				my $digest   = new $hash(@param);
 				my $function = sub {
-					my $digest = new $hash(@param);
 					my $hmac = new Digest::HMAC( shift, $digest, @block );
 					$hmac->add(shift);
 					return $hmac->digest;
@@ -495,7 +535,7 @@ sub vrfyerrstr {
 		my $private = shift;	# closure keeps private key private
 		$keyref->{key} = sub {
 			my $function = $keyref->{digest};
-			return &$function( $private, shift );
+			return &$function( $private, @_ );
 		};
 		return undef;
 	}
@@ -504,11 +544,12 @@ sub vrfyerrstr {
 	sub _mac_function {		## apply keyed hash function to argument
 		my $self = shift;
 
-		my $keyref = $keytable{$self->{owner}->canonical} ||= {};
-		$self->algorithm( $self->algorithm ) unless $keyref->{digest};
-		$keyref->{digest} ||= $self->sig_function;
+		my $owner = $self->{owner}->canonical;
+		$self->algorithm( $self->algorithm ) unless $keytable{$owner}{digest};
+		my $keyref = $keytable{$owner};
+		$keyref->{digest} = $self->sig_function unless $keyref->{digest};
 		my $function = $keyref->{key};
-		return &$function(shift);
+		&$function(@_);
 	}
 }
 
@@ -518,7 +559,8 @@ sub vrfyerrstr {
 
 sub _chain {
 	my $self = shift;
-	return bless {%$self, macbin => undef, link => $self}, ref($self);
+	$self->{link} = undef;
+	bless {%$self, link => $self}, ref($self);
 }
 
 
@@ -529,12 +571,12 @@ __END__
 =head1 SYNOPSIS
 
     use Net::DNS;
+    $tsig = create Net::DNS::RR::TSIG( $keyfile );
+
     $tsig = create Net::DNS::RR::TSIG( $keyfile,
 					fudge => 300
 					);
 
-    $tsig = create Net::DNS::RR::TSIG( $keyname, $key );
-
 =head1 DESCRIPTION
 
 Class for DNS Transaction Signature (TSIG) resource records.
@@ -639,6 +681,8 @@ The message ID from the header of the original packet.
 
 =head2 error
 
+=head2 vrfyerrstr
+
      $rcode = $tsig->error;
 
 Returns the RCODE covering TSIG processing.  Common values are
@@ -647,8 +691,7 @@ NOERROR, BADSIG, BADKEY, and BADTIME.  See RFC 2845 for details.
 
 =head2 other
 
-    $other = $rr->other;
-    $rr->other( $other );
+     $other = $tsig->other;
 
 This field should be empty unless the error is BADTIME, in which
 case it will contain the server time as the number of seconds since
@@ -687,11 +730,11 @@ needed for TSIG-GSS.
 					fudge => 300
 					);
 
-    $tsig = create Net::DNS::RR::TSIG( $keyname, $key );
-
 Returns a TSIG RR constructed using the parameters in the specified
 key file, which is assumed to have been generated by dnssec-keygen.
 
+    $tsig = create Net::DNS::RR::TSIG( $keyname, $key );
+
 The two argument form is supported for backward compatibility.
 
 =head2 verify
@@ -768,22 +811,40 @@ before the code stops working on 7 February 2106.
 
 =head1 COPYRIGHT
 
-Copyright (c)2002 Michael Fuhr. 
+Copyright (c)2000,2001 Michael Fuhr. 
 
-Portions Copyright (c)2002-2004 Chris Reinhardt.
+Portions Copyright (c)2002,2003 Chris Reinhardt.
 
 Portions Copyright (c)2013 Dick Franks.
 
 All rights reserved.
 
-This program is free software; you may redistribute it and/or
-modify it under the same terms as Perl itself.
-
 Package template (c)2009,2012 O.M.Kolkman and R.W.Franks.
 
 
+=head1 LICENSE
+
+Permission to use, copy, modify, and distribute this software and its
+documentation for any purpose and without fee is hereby granted, provided
+that the above copyright notice appear in all copies and that both that
+copyright notice and this permission notice appear in supporting
+documentation, and that the name of the author not be used in advertising
+or publicity pertaining to distribution of the software without specific
+prior written permission.
+
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+DEALINGS IN THE SOFTWARE.
+
+
 =head1 SEE ALSO
 
 L<perl>, L<Net::DNS>, L<Net::DNS::RR>, RFC2845, RFC4635
 
+L<TSIG Algorithm Names|http://www.iana.org/assignments/tsig-algorithm-names>
+
 =cut
